# Generated by cloudcoil-model-codegen v0.3.0
# DO NOT EDIT

from __future__ import annotations

from typing import (
    Annotated,
    Callable,
    Dict,
    List,
    Literal,
    Optional,
    Type,
    overload,
)

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil.pydantic import (
    BaseModel,
    BaseModelBuilder,
    BuilderContextBase,
    GenericListBuilder,
    ListBuilderContext,
    Never,
    Self,
)
from cloudcoil.resources import Resource


class NonResourceAttributes(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["NonResourceAttributes"]:
            return NonResourceAttributes

        def build(self) -> "NonResourceAttributes":
            return NonResourceAttributes(**self._attrs)

        def path(self, value: Optional[str], /) -> Self:
            """
            Path is the URL path of the request
            """
            return self._set("path", value)

        def verb(self, value: Optional[str], /) -> Self:
            """
            Verb is the standard HTTP verb
            """
            return self._set("verb", value)

    class BuilderContext(BuilderContextBase["NonResourceAttributes.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = NonResourceAttributes.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for NonResourceAttributes."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["NonResourceAttributes", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use NonResourceAttributes.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    path: Optional[str] = None
    """
    Path is the URL path of the request
    """
    verb: Optional[str] = None
    """
    Verb is the standard HTTP verb
    """


class NonResourceRule(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["NonResourceRule"]:
            return NonResourceRule

        def build(self) -> "NonResourceRule":
            return NonResourceRule(**self._attrs)

        def non_resource_ur_ls(self, value: Optional[List[str]], /) -> Self:
            """
            NonResourceURLs is a set of partial urls that a user should have access to.  *s are allowed, but only as the full, final step in the path.  "*" means all.
            """
            return self._set("non_resource_ur_ls", value)

        def verbs(self, value: List[str], /) -> Self:
            """
            Verb is a list of kubernetes non-resource API verbs, like: get, post, put, delete, patch, head, options.  "*" means all.
            """
            return self._set("verbs", value)

    class BuilderContext(BuilderContextBase["NonResourceRule.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = NonResourceRule.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for NonResourceRule."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["NonResourceRule", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use NonResourceRule.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    non_resource_ur_ls: Annotated[Optional[List[str]], Field(alias="nonResourceURLs")] = None
    """
    NonResourceURLs is a set of partial urls that a user should have access to.  *s are allowed, but only as the full, final step in the path.  "*" means all.
    """
    verbs: List[str]
    """
    Verb is a list of kubernetes non-resource API verbs, like: get, post, put, delete, patch, head, options.  "*" means all.
    """


class ResourceAttributes(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ResourceAttributes"]:
            return ResourceAttributes

        def build(self) -> "ResourceAttributes":
            return ResourceAttributes(**self._attrs)

        def group(self, value: Optional[str], /) -> Self:
            """
            Group is the API Group of the Resource.  "*" means all.
            """
            return self._set("group", value)

        def name(self, value: Optional[str], /) -> Self:
            """
            Name is the name of the resource being requested for a "get" or deleted for a "delete". "" (empty) means all.
            """
            return self._set("name", value)

        def namespace(self, value: Optional[str], /) -> Self:
            """
            Namespace is the namespace of the action being requested.  Currently, there is no distinction between no namespace and all namespaces "" (empty) is defaulted for LocalSubjectAccessReviews "" (empty) is empty for cluster-scoped resources "" (empty) means "all" for namespace scoped resources from a SubjectAccessReview or SelfSubjectAccessReview
            """
            return self._set("namespace", value)

        def resource(self, value: Optional[str], /) -> Self:
            """
            Resource is one of the existing resource types.  "*" means all.
            """
            return self._set("resource", value)

        def subresource(self, value: Optional[str], /) -> Self:
            """
            Subresource is one of the existing resource types.  "" means none.
            """
            return self._set("subresource", value)

        def verb(self, value: Optional[str], /) -> Self:
            """
            Verb is a kubernetes resource API verb, like: get, list, watch, create, update, delete, proxy.  "*" means all.
            """
            return self._set("verb", value)

        def version(self, value: Optional[str], /) -> Self:
            """
            Version is the API Version of the Resource.  "*" means all.
            """
            return self._set("version", value)

    class BuilderContext(BuilderContextBase["ResourceAttributes.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ResourceAttributes.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ResourceAttributes."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ResourceAttributes", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ResourceAttributes.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    group: Optional[str] = None
    """
    Group is the API Group of the Resource.  "*" means all.
    """
    name: Optional[str] = None
    """
    Name is the name of the resource being requested for a "get" or deleted for a "delete". "" (empty) means all.
    """
    namespace: Optional[str] = None
    """
    Namespace is the namespace of the action being requested.  Currently, there is no distinction between no namespace and all namespaces "" (empty) is defaulted for LocalSubjectAccessReviews "" (empty) is empty for cluster-scoped resources "" (empty) means "all" for namespace scoped resources from a SubjectAccessReview or SelfSubjectAccessReview
    """
    resource: Optional[str] = None
    """
    Resource is one of the existing resource types.  "*" means all.
    """
    subresource: Optional[str] = None
    """
    Subresource is one of the existing resource types.  "" means none.
    """
    verb: Optional[str] = None
    """
    Verb is a kubernetes resource API verb, like: get, list, watch, create, update, delete, proxy.  "*" means all.
    """
    version: Optional[str] = None
    """
    Version is the API Version of the Resource.  "*" means all.
    """


class ResourceRule(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ResourceRule"]:
            return ResourceRule

        def build(self) -> "ResourceRule":
            return ResourceRule(**self._attrs)

        def api_groups(self, value: Optional[List[str]], /) -> Self:
            """
            APIGroups is the name of the APIGroup that contains the resources.  If multiple API groups are specified, any action requested against one of the enumerated resources in any API group will be allowed.  "*" means all.
            """
            return self._set("api_groups", value)

        def resource_names(self, value: Optional[List[str]], /) -> Self:
            """
            ResourceNames is an optional white list of names that the rule applies to.  An empty set means that everything is allowed.  "*" means all.
            """
            return self._set("resource_names", value)

        def resources(self, value: Optional[List[str]], /) -> Self:
            """
            Resources is a list of resources this rule applies to.  "*" means all in the specified apiGroups.
             "*/foo" represents the subresource 'foo' for all resources in the specified apiGroups.
            """
            return self._set("resources", value)

        def verbs(self, value: List[str], /) -> Self:
            """
            Verb is a list of kubernetes resource API verbs, like: get, list, watch, create, update, delete, proxy.  "*" means all.
            """
            return self._set("verbs", value)

    class BuilderContext(BuilderContextBase["ResourceRule.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ResourceRule.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ResourceRule."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ResourceRule", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ResourceRule.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_groups: Annotated[Optional[List[str]], Field(alias="apiGroups")] = None
    """
    APIGroups is the name of the APIGroup that contains the resources.  If multiple API groups are specified, any action requested against one of the enumerated resources in any API group will be allowed.  "*" means all.
    """
    resource_names: Annotated[Optional[List[str]], Field(alias="resourceNames")] = None
    """
    ResourceNames is an optional white list of names that the rule applies to.  An empty set means that everything is allowed.  "*" means all.
    """
    resources: Optional[List[str]] = None
    """
    Resources is a list of resources this rule applies to.  "*" means all in the specified apiGroups.
     "*/foo" represents the subresource 'foo' for all resources in the specified apiGroups.
    """
    verbs: List[str]
    """
    Verb is a list of kubernetes resource API verbs, like: get, list, watch, create, update, delete, proxy.  "*" means all.
    """


class SelfSubjectAccessReviewSpec(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["SelfSubjectAccessReviewSpec"]:
            return SelfSubjectAccessReviewSpec

        def build(self) -> "SelfSubjectAccessReviewSpec":
            return SelfSubjectAccessReviewSpec(**self._attrs)

        @overload
        def non_resource_attributes(
            self, value_or_callback: Optional[NonResourceAttributes], /
        ) -> "SelfSubjectAccessReviewSpec.Builder": ...

        @overload
        def non_resource_attributes(
            self,
            value_or_callback: Callable[
                [NonResourceAttributes.Builder],
                NonResourceAttributes.Builder | NonResourceAttributes,
            ],
            /,
        ) -> "SelfSubjectAccessReviewSpec.Builder": ...

        @overload
        def non_resource_attributes(
            self, value_or_callback: Never = ...
        ) -> "NonResourceAttributes.BuilderContext": ...

        def non_resource_attributes(self, value_or_callback=None, /):
            """
            NonResourceAttributes describes information for a non-resource access request
            """
            if self._in_context and value_or_callback is None:
                context = NonResourceAttributes.BuilderContext()
                context._parent_builder = self
                context._field_name = "non_resource_attributes"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(NonResourceAttributes.builder())
                if isinstance(output, NonResourceAttributes.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("non_resource_attributes", value)

        @overload
        def resource_attributes(
            self, value_or_callback: Optional[ResourceAttributes], /
        ) -> "SelfSubjectAccessReviewSpec.Builder": ...

        @overload
        def resource_attributes(
            self,
            value_or_callback: Callable[
                [ResourceAttributes.Builder],
                ResourceAttributes.Builder | ResourceAttributes,
            ],
            /,
        ) -> "SelfSubjectAccessReviewSpec.Builder": ...

        @overload
        def resource_attributes(
            self, value_or_callback: Never = ...
        ) -> "ResourceAttributes.BuilderContext": ...

        def resource_attributes(self, value_or_callback=None, /):
            """
            ResourceAuthorizationAttributes describes information for a resource access request
            """
            if self._in_context and value_or_callback is None:
                context = ResourceAttributes.BuilderContext()
                context._parent_builder = self
                context._field_name = "resource_attributes"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ResourceAttributes.builder())
                if isinstance(output, ResourceAttributes.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("resource_attributes", value)

    class BuilderContext(BuilderContextBase["SelfSubjectAccessReviewSpec.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = SelfSubjectAccessReviewSpec.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for SelfSubjectAccessReviewSpec."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["SelfSubjectAccessReviewSpec", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use SelfSubjectAccessReviewSpec.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    non_resource_attributes: Annotated[
        Optional[NonResourceAttributes], Field(alias="nonResourceAttributes")
    ] = None
    """
    NonResourceAttributes describes information for a non-resource access request
    """
    resource_attributes: Annotated[
        Optional[ResourceAttributes], Field(alias="resourceAttributes")
    ] = None
    """
    ResourceAuthorizationAttributes describes information for a resource access request
    """


class SelfSubjectRulesReviewSpec(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["SelfSubjectRulesReviewSpec"]:
            return SelfSubjectRulesReviewSpec

        def build(self) -> "SelfSubjectRulesReviewSpec":
            return SelfSubjectRulesReviewSpec(**self._attrs)

        def namespace(self, value: Optional[str], /) -> Self:
            """
            Namespace to evaluate rules for. Required.
            """
            return self._set("namespace", value)

    class BuilderContext(BuilderContextBase["SelfSubjectRulesReviewSpec.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = SelfSubjectRulesReviewSpec.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for SelfSubjectRulesReviewSpec."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["SelfSubjectRulesReviewSpec", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use SelfSubjectRulesReviewSpec.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    namespace: Optional[str] = None
    """
    Namespace to evaluate rules for. Required.
    """


class SubjectAccessReviewSpec(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["SubjectAccessReviewSpec"]:
            return SubjectAccessReviewSpec

        def build(self) -> "SubjectAccessReviewSpec":
            return SubjectAccessReviewSpec(**self._attrs)

        def extra(self, value: Optional[Dict[str, List[str]]], /) -> Self:
            """
            Extra corresponds to the user.Info.GetExtra() method from the authenticator.  Since that is input to the authorizer it needs a reflection here.
            """
            return self._set("extra", value)

        def groups(self, value: Optional[List[str]], /) -> Self:
            """
            Groups is the groups you're testing for.
            """
            return self._set("groups", value)

        @overload
        def non_resource_attributes(
            self, value_or_callback: Optional[NonResourceAttributes], /
        ) -> "SubjectAccessReviewSpec.Builder": ...

        @overload
        def non_resource_attributes(
            self,
            value_or_callback: Callable[
                [NonResourceAttributes.Builder],
                NonResourceAttributes.Builder | NonResourceAttributes,
            ],
            /,
        ) -> "SubjectAccessReviewSpec.Builder": ...

        @overload
        def non_resource_attributes(
            self, value_or_callback: Never = ...
        ) -> "NonResourceAttributes.BuilderContext": ...

        def non_resource_attributes(self, value_or_callback=None, /):
            """
            NonResourceAttributes describes information for a non-resource access request
            """
            if self._in_context and value_or_callback is None:
                context = NonResourceAttributes.BuilderContext()
                context._parent_builder = self
                context._field_name = "non_resource_attributes"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(NonResourceAttributes.builder())
                if isinstance(output, NonResourceAttributes.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("non_resource_attributes", value)

        @overload
        def resource_attributes(
            self, value_or_callback: Optional[ResourceAttributes], /
        ) -> "SubjectAccessReviewSpec.Builder": ...

        @overload
        def resource_attributes(
            self,
            value_or_callback: Callable[
                [ResourceAttributes.Builder],
                ResourceAttributes.Builder | ResourceAttributes,
            ],
            /,
        ) -> "SubjectAccessReviewSpec.Builder": ...

        @overload
        def resource_attributes(
            self, value_or_callback: Never = ...
        ) -> "ResourceAttributes.BuilderContext": ...

        def resource_attributes(self, value_or_callback=None, /):
            """
            ResourceAuthorizationAttributes describes information for a resource access request
            """
            if self._in_context and value_or_callback is None:
                context = ResourceAttributes.BuilderContext()
                context._parent_builder = self
                context._field_name = "resource_attributes"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ResourceAttributes.builder())
                if isinstance(output, ResourceAttributes.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("resource_attributes", value)

        def uid(self, value: Optional[str], /) -> Self:
            """
            UID information about the requesting user.
            """
            return self._set("uid", value)

        def user(self, value: Optional[str], /) -> Self:
            """
            User is the user you're testing for. If you specify "User" but not "Groups", then is it interpreted as "What if User were not a member of any groups
            """
            return self._set("user", value)

    class BuilderContext(BuilderContextBase["SubjectAccessReviewSpec.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = SubjectAccessReviewSpec.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for SubjectAccessReviewSpec."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["SubjectAccessReviewSpec", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use SubjectAccessReviewSpec.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    extra: Optional[Dict[str, List[str]]] = None
    """
    Extra corresponds to the user.Info.GetExtra() method from the authenticator.  Since that is input to the authorizer it needs a reflection here.
    """
    groups: Optional[List[str]] = None
    """
    Groups is the groups you're testing for.
    """
    non_resource_attributes: Annotated[
        Optional[NonResourceAttributes], Field(alias="nonResourceAttributes")
    ] = None
    """
    NonResourceAttributes describes information for a non-resource access request
    """
    resource_attributes: Annotated[
        Optional[ResourceAttributes], Field(alias="resourceAttributes")
    ] = None
    """
    ResourceAuthorizationAttributes describes information for a resource access request
    """
    uid: Optional[str] = None
    """
    UID information about the requesting user.
    """
    user: Optional[str] = None
    """
    User is the user you're testing for. If you specify "User" but not "Groups", then is it interpreted as "What if User were not a member of any groups
    """


class SubjectAccessReviewStatus(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["SubjectAccessReviewStatus"]:
            return SubjectAccessReviewStatus

        def build(self) -> "SubjectAccessReviewStatus":
            return SubjectAccessReviewStatus(**self._attrs)

        def allowed(self, value: bool, /) -> Self:
            """
            Allowed is required. True if the action would be allowed, false otherwise.
            """
            return self._set("allowed", value)

        def denied(self, value: Optional[bool], /) -> Self:
            """
            Denied is optional. True if the action would be denied, otherwise false. If both allowed is false and denied is false, then the authorizer has no opinion on whether to authorize the action. Denied may not be true if Allowed is true.
            """
            return self._set("denied", value)

        def evaluation_error(self, value: Optional[str], /) -> Self:
            """
            EvaluationError is an indication that some error occurred during the authorization check. It is entirely possible to get an error and be able to continue determine authorization status in spite of it. For instance, RBAC can be missing a role, but enough roles are still present and bound to reason about the request.
            """
            return self._set("evaluation_error", value)

        def reason(self, value: Optional[str], /) -> Self:
            """
            Reason is optional.  It indicates why a request was allowed or denied.
            """
            return self._set("reason", value)

    class BuilderContext(BuilderContextBase["SubjectAccessReviewStatus.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = SubjectAccessReviewStatus.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for SubjectAccessReviewStatus."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["SubjectAccessReviewStatus", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use SubjectAccessReviewStatus.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    allowed: bool
    """
    Allowed is required. True if the action would be allowed, false otherwise.
    """
    denied: Optional[bool] = None
    """
    Denied is optional. True if the action would be denied, otherwise false. If both allowed is false and denied is false, then the authorizer has no opinion on whether to authorize the action. Denied may not be true if Allowed is true.
    """
    evaluation_error: Annotated[Optional[str], Field(alias="evaluationError")] = None
    """
    EvaluationError is an indication that some error occurred during the authorization check. It is entirely possible to get an error and be able to continue determine authorization status in spite of it. For instance, RBAC can be missing a role, but enough roles are still present and bound to reason about the request.
    """
    reason: Optional[str] = None
    """
    Reason is optional.  It indicates why a request was allowed or denied.
    """


class SubjectRulesReviewStatus(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["SubjectRulesReviewStatus"]:
            return SubjectRulesReviewStatus

        def build(self) -> "SubjectRulesReviewStatus":
            return SubjectRulesReviewStatus(**self._attrs)

        def evaluation_error(self, value: Optional[str], /) -> Self:
            """
            EvaluationError can appear in combination with Rules. It indicates an error occurred during rule evaluation, such as an authorizer that doesn't support rule evaluation, and that ResourceRules and/or NonResourceRules may be incomplete.
            """
            return self._set("evaluation_error", value)

        def incomplete(self, value: bool, /) -> Self:
            """
            Incomplete is true when the rules returned by this call are incomplete. This is most commonly encountered when an authorizer, such as an external authorizer, doesn't support rules evaluation.
            """
            return self._set("incomplete", value)

        @overload
        def non_resource_rules(
            self, value_or_callback: List[NonResourceRule], /
        ) -> "SubjectRulesReviewStatus.Builder": ...

        @overload
        def non_resource_rules(
            self,
            value_or_callback: Callable[
                [GenericListBuilder[NonResourceRule, NonResourceRule.Builder]],
                GenericListBuilder[NonResourceRule, NonResourceRule.Builder]
                | List[NonResourceRule],
            ],
            /,
        ) -> "SubjectRulesReviewStatus.Builder": ...

        @overload
        def non_resource_rules(
            self, value_or_callback: Never = ...
        ) -> ListBuilderContext[NonResourceRule.Builder]: ...

        def non_resource_rules(self, value_or_callback=None, /):
            """
            NonResourceRules is the list of actions the subject is allowed to perform on non-resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.
            """
            if self._in_context and value_or_callback is None:
                context = ListBuilderContext[NonResourceRule.Builder]()
                context._parent_builder = self
                context._field_name = "non_resource_rules"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(NonResourceRule.list_builder())
                if isinstance(output, GenericListBuilder):
                    value = output.build()
                else:
                    value = output
            return self._set("non_resource_rules", value)

        @overload
        def resource_rules(
            self, value_or_callback: List[ResourceRule], /
        ) -> "SubjectRulesReviewStatus.Builder": ...

        @overload
        def resource_rules(
            self,
            value_or_callback: Callable[
                [GenericListBuilder[ResourceRule, ResourceRule.Builder]],
                GenericListBuilder[ResourceRule, ResourceRule.Builder] | List[ResourceRule],
            ],
            /,
        ) -> "SubjectRulesReviewStatus.Builder": ...

        @overload
        def resource_rules(
            self, value_or_callback: Never = ...
        ) -> ListBuilderContext[ResourceRule.Builder]: ...

        def resource_rules(self, value_or_callback=None, /):
            """
            ResourceRules is the list of actions the subject is allowed to perform on resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.
            """
            if self._in_context and value_or_callback is None:
                context = ListBuilderContext[ResourceRule.Builder]()
                context._parent_builder = self
                context._field_name = "resource_rules"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ResourceRule.list_builder())
                if isinstance(output, GenericListBuilder):
                    value = output.build()
                else:
                    value = output
            return self._set("resource_rules", value)

    class BuilderContext(BuilderContextBase["SubjectRulesReviewStatus.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = SubjectRulesReviewStatus.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for SubjectRulesReviewStatus."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["SubjectRulesReviewStatus", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use SubjectRulesReviewStatus.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    evaluation_error: Annotated[Optional[str], Field(alias="evaluationError")] = None
    """
    EvaluationError can appear in combination with Rules. It indicates an error occurred during rule evaluation, such as an authorizer that doesn't support rule evaluation, and that ResourceRules and/or NonResourceRules may be incomplete.
    """
    incomplete: bool
    """
    Incomplete is true when the rules returned by this call are incomplete. This is most commonly encountered when an authorizer, such as an external authorizer, doesn't support rules evaluation.
    """
    non_resource_rules: Annotated[List[NonResourceRule], Field(alias="nonResourceRules")]
    """
    NonResourceRules is the list of actions the subject is allowed to perform on non-resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.
    """
    resource_rules: Annotated[List[ResourceRule], Field(alias="resourceRules")]
    """
    ResourceRules is the list of actions the subject is allowed to perform on resources. The list ordering isn't significant, may contain duplicates, and possibly be incomplete.
    """


class LocalSubjectAccessReview(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["LocalSubjectAccessReview"]:
            return LocalSubjectAccessReview

        def build(self) -> "LocalSubjectAccessReview":
            return LocalSubjectAccessReview(**self._attrs)

        def api_version(self, value: Optional[Literal["authorization.k8s.io/v1"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["LocalSubjectAccessReview"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "LocalSubjectAccessReview.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "LocalSubjectAccessReview.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def spec(
            self, value_or_callback: SubjectAccessReviewSpec, /
        ) -> "LocalSubjectAccessReview.Builder": ...

        @overload
        def spec(
            self,
            value_or_callback: Callable[
                [SubjectAccessReviewSpec.Builder],
                SubjectAccessReviewSpec.Builder | SubjectAccessReviewSpec,
            ],
            /,
        ) -> "LocalSubjectAccessReview.Builder": ...

        @overload
        def spec(
            self, value_or_callback: Never = ...
        ) -> "SubjectAccessReviewSpec.BuilderContext": ...

        def spec(self, value_or_callback=None, /):
            """
            Spec holds information about the request being evaluated.  spec.namespace must be equal to the namespace you made the request against.  If empty, it is defaulted.
            """
            if self._in_context and value_or_callback is None:
                context = SubjectAccessReviewSpec.BuilderContext()
                context._parent_builder = self
                context._field_name = "spec"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(SubjectAccessReviewSpec.builder())
                if isinstance(output, SubjectAccessReviewSpec.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("spec", value)

        @overload
        def status(
            self, value_or_callback: Optional[SubjectAccessReviewStatus], /
        ) -> "LocalSubjectAccessReview.Builder": ...

        @overload
        def status(
            self,
            value_or_callback: Callable[
                [SubjectAccessReviewStatus.Builder],
                SubjectAccessReviewStatus.Builder | SubjectAccessReviewStatus,
            ],
            /,
        ) -> "LocalSubjectAccessReview.Builder": ...

        @overload
        def status(
            self, value_or_callback: Never = ...
        ) -> "SubjectAccessReviewStatus.BuilderContext": ...

        def status(self, value_or_callback=None, /):
            """
            Status is filled in by the server and indicates whether the request is allowed or not
            """
            if self._in_context and value_or_callback is None:
                context = SubjectAccessReviewStatus.BuilderContext()
                context._parent_builder = self
                context._field_name = "status"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(SubjectAccessReviewStatus.builder())
                if isinstance(output, SubjectAccessReviewStatus.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("status", value)

    class BuilderContext(BuilderContextBase["LocalSubjectAccessReview.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = LocalSubjectAccessReview.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for LocalSubjectAccessReview."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["LocalSubjectAccessReview", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use LocalSubjectAccessReview.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[
        Optional[Literal["authorization.k8s.io/v1"]], Field(alias="apiVersion")
    ] = "authorization.k8s.io/v1"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["LocalSubjectAccessReview"]] = "LocalSubjectAccessReview"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    spec: SubjectAccessReviewSpec
    """
    Spec holds information about the request being evaluated.  spec.namespace must be equal to the namespace you made the request against.  If empty, it is defaulted.
    """
    status: Optional[SubjectAccessReviewStatus] = None
    """
    Status is filled in by the server and indicates whether the request is allowed or not
    """


class SelfSubjectAccessReview(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["SelfSubjectAccessReview"]:
            return SelfSubjectAccessReview

        def build(self) -> "SelfSubjectAccessReview":
            return SelfSubjectAccessReview(**self._attrs)

        def api_version(self, value: Optional[Literal["authorization.k8s.io/v1"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["SelfSubjectAccessReview"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "SelfSubjectAccessReview.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "SelfSubjectAccessReview.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def spec(
            self, value_or_callback: SelfSubjectAccessReviewSpec, /
        ) -> "SelfSubjectAccessReview.Builder": ...

        @overload
        def spec(
            self,
            value_or_callback: Callable[
                [SelfSubjectAccessReviewSpec.Builder],
                SelfSubjectAccessReviewSpec.Builder | SelfSubjectAccessReviewSpec,
            ],
            /,
        ) -> "SelfSubjectAccessReview.Builder": ...

        @overload
        def spec(
            self, value_or_callback: Never = ...
        ) -> "SelfSubjectAccessReviewSpec.BuilderContext": ...

        def spec(self, value_or_callback=None, /):
            """
            Spec holds information about the request being evaluated.  user and groups must be empty
            """
            if self._in_context and value_or_callback is None:
                context = SelfSubjectAccessReviewSpec.BuilderContext()
                context._parent_builder = self
                context._field_name = "spec"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(SelfSubjectAccessReviewSpec.builder())
                if isinstance(output, SelfSubjectAccessReviewSpec.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("spec", value)

        @overload
        def status(
            self, value_or_callback: Optional[SubjectAccessReviewStatus], /
        ) -> "SelfSubjectAccessReview.Builder": ...

        @overload
        def status(
            self,
            value_or_callback: Callable[
                [SubjectAccessReviewStatus.Builder],
                SubjectAccessReviewStatus.Builder | SubjectAccessReviewStatus,
            ],
            /,
        ) -> "SelfSubjectAccessReview.Builder": ...

        @overload
        def status(
            self, value_or_callback: Never = ...
        ) -> "SubjectAccessReviewStatus.BuilderContext": ...

        def status(self, value_or_callback=None, /):
            """
            Status is filled in by the server and indicates whether the request is allowed or not
            """
            if self._in_context and value_or_callback is None:
                context = SubjectAccessReviewStatus.BuilderContext()
                context._parent_builder = self
                context._field_name = "status"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(SubjectAccessReviewStatus.builder())
                if isinstance(output, SubjectAccessReviewStatus.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("status", value)

    class BuilderContext(BuilderContextBase["SelfSubjectAccessReview.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = SelfSubjectAccessReview.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for SelfSubjectAccessReview."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["SelfSubjectAccessReview", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use SelfSubjectAccessReview.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[
        Optional[Literal["authorization.k8s.io/v1"]], Field(alias="apiVersion")
    ] = "authorization.k8s.io/v1"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["SelfSubjectAccessReview"]] = "SelfSubjectAccessReview"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    spec: SelfSubjectAccessReviewSpec
    """
    Spec holds information about the request being evaluated.  user and groups must be empty
    """
    status: Optional[SubjectAccessReviewStatus] = None
    """
    Status is filled in by the server and indicates whether the request is allowed or not
    """


class SelfSubjectRulesReview(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["SelfSubjectRulesReview"]:
            return SelfSubjectRulesReview

        def build(self) -> "SelfSubjectRulesReview":
            return SelfSubjectRulesReview(**self._attrs)

        def api_version(self, value: Optional[Literal["authorization.k8s.io/v1"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["SelfSubjectRulesReview"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "SelfSubjectRulesReview.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "SelfSubjectRulesReview.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def spec(
            self, value_or_callback: SelfSubjectRulesReviewSpec, /
        ) -> "SelfSubjectRulesReview.Builder": ...

        @overload
        def spec(
            self,
            value_or_callback: Callable[
                [SelfSubjectRulesReviewSpec.Builder],
                SelfSubjectRulesReviewSpec.Builder | SelfSubjectRulesReviewSpec,
            ],
            /,
        ) -> "SelfSubjectRulesReview.Builder": ...

        @overload
        def spec(
            self, value_or_callback: Never = ...
        ) -> "SelfSubjectRulesReviewSpec.BuilderContext": ...

        def spec(self, value_or_callback=None, /):
            """
            Spec holds information about the request being evaluated.
            """
            if self._in_context and value_or_callback is None:
                context = SelfSubjectRulesReviewSpec.BuilderContext()
                context._parent_builder = self
                context._field_name = "spec"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(SelfSubjectRulesReviewSpec.builder())
                if isinstance(output, SelfSubjectRulesReviewSpec.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("spec", value)

        @overload
        def status(
            self, value_or_callback: Optional[SubjectRulesReviewStatus], /
        ) -> "SelfSubjectRulesReview.Builder": ...

        @overload
        def status(
            self,
            value_or_callback: Callable[
                [SubjectRulesReviewStatus.Builder],
                SubjectRulesReviewStatus.Builder | SubjectRulesReviewStatus,
            ],
            /,
        ) -> "SelfSubjectRulesReview.Builder": ...

        @overload
        def status(
            self, value_or_callback: Never = ...
        ) -> "SubjectRulesReviewStatus.BuilderContext": ...

        def status(self, value_or_callback=None, /):
            """
            Status is filled in by the server and indicates the set of actions a user can perform.
            """
            if self._in_context and value_or_callback is None:
                context = SubjectRulesReviewStatus.BuilderContext()
                context._parent_builder = self
                context._field_name = "status"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(SubjectRulesReviewStatus.builder())
                if isinstance(output, SubjectRulesReviewStatus.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("status", value)

    class BuilderContext(BuilderContextBase["SelfSubjectRulesReview.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = SelfSubjectRulesReview.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for SelfSubjectRulesReview."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["SelfSubjectRulesReview", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use SelfSubjectRulesReview.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[
        Optional[Literal["authorization.k8s.io/v1"]], Field(alias="apiVersion")
    ] = "authorization.k8s.io/v1"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["SelfSubjectRulesReview"]] = "SelfSubjectRulesReview"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    spec: SelfSubjectRulesReviewSpec
    """
    Spec holds information about the request being evaluated.
    """
    status: Optional[SubjectRulesReviewStatus] = None
    """
    Status is filled in by the server and indicates the set of actions a user can perform.
    """


class SubjectAccessReview(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["SubjectAccessReview"]:
            return SubjectAccessReview

        def build(self) -> "SubjectAccessReview":
            return SubjectAccessReview(**self._attrs)

        def api_version(self, value: Optional[Literal["authorization.k8s.io/v1"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["SubjectAccessReview"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "SubjectAccessReview.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "SubjectAccessReview.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def spec(
            self, value_or_callback: SubjectAccessReviewSpec, /
        ) -> "SubjectAccessReview.Builder": ...

        @overload
        def spec(
            self,
            value_or_callback: Callable[
                [SubjectAccessReviewSpec.Builder],
                SubjectAccessReviewSpec.Builder | SubjectAccessReviewSpec,
            ],
            /,
        ) -> "SubjectAccessReview.Builder": ...

        @overload
        def spec(
            self, value_or_callback: Never = ...
        ) -> "SubjectAccessReviewSpec.BuilderContext": ...

        def spec(self, value_or_callback=None, /):
            """
            Spec holds information about the request being evaluated
            """
            if self._in_context and value_or_callback is None:
                context = SubjectAccessReviewSpec.BuilderContext()
                context._parent_builder = self
                context._field_name = "spec"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(SubjectAccessReviewSpec.builder())
                if isinstance(output, SubjectAccessReviewSpec.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("spec", value)

        @overload
        def status(
            self, value_or_callback: Optional[SubjectAccessReviewStatus], /
        ) -> "SubjectAccessReview.Builder": ...

        @overload
        def status(
            self,
            value_or_callback: Callable[
                [SubjectAccessReviewStatus.Builder],
                SubjectAccessReviewStatus.Builder | SubjectAccessReviewStatus,
            ],
            /,
        ) -> "SubjectAccessReview.Builder": ...

        @overload
        def status(
            self, value_or_callback: Never = ...
        ) -> "SubjectAccessReviewStatus.BuilderContext": ...

        def status(self, value_or_callback=None, /):
            """
            Status is filled in by the server and indicates whether the request is allowed or not
            """
            if self._in_context and value_or_callback is None:
                context = SubjectAccessReviewStatus.BuilderContext()
                context._parent_builder = self
                context._field_name = "status"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(SubjectAccessReviewStatus.builder())
                if isinstance(output, SubjectAccessReviewStatus.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("status", value)

    class BuilderContext(BuilderContextBase["SubjectAccessReview.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = SubjectAccessReview.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for SubjectAccessReview."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["SubjectAccessReview", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use SubjectAccessReview.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[
        Optional[Literal["authorization.k8s.io/v1"]], Field(alias="apiVersion")
    ] = "authorization.k8s.io/v1"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["SubjectAccessReview"]] = "SubjectAccessReview"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    spec: SubjectAccessReviewSpec
    """
    Spec holds information about the request being evaluated
    """
    status: Optional[SubjectAccessReviewStatus] = None
    """
    Status is filled in by the server and indicates whether the request is allowed or not
    """



# Generated by cloudcoil-model-codegen v0.3.0
# DO NOT EDIT

from __future__ import annotations

from typing import (
    Annotated,
    Callable,
    List,
    Literal,
    Optional,
    Type,
    overload,
)

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil.pydantic import (
    BaseModel,
    BaseModelBuilder,
    BuilderContextBase,
    GenericListBuilder,
    ListBuilderContext,
    Never,
    Self,
)
from cloudcoil.resources import Resource, ResourceList


class APIServiceCondition(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["APIServiceCondition"]:
            return APIServiceCondition

        def build(self) -> "APIServiceCondition":
            return APIServiceCondition(**self._attrs)

        @overload
        def last_transition_time(
            self, value_or_callback: Optional[apimachinery.Time], /
        ) -> "APIServiceCondition.Builder": ...

        @overload
        def last_transition_time(
            self,
            value_or_callback: Callable[
                [apimachinery.Time.Builder],
                apimachinery.Time.Builder | apimachinery.Time,
            ],
            /,
        ) -> "APIServiceCondition.Builder": ...

        @overload
        def last_transition_time(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.Time.BuilderContext": ...

        def last_transition_time(self, value_or_callback=None, /):
            """
            Last time the condition transitioned from one status to another.
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.Time.BuilderContext()
                context._parent_builder = self
                context._field_name = "last_transition_time"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.Time.builder())
                if isinstance(output, apimachinery.Time.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("last_transition_time", value)

        def message(self, value: Optional[str], /) -> Self:
            """
            Human-readable message indicating details about last transition.
            """
            return self._set("message", value)

        def reason(self, value: Optional[str], /) -> Self:
            """
            Unique, one-word, CamelCase reason for the condition's last transition.
            """
            return self._set("reason", value)

        def status(self, value: str, /) -> Self:
            """
            Status is the status of the condition. Can be True, False, Unknown.
            """
            return self._set("status", value)

        def type(self, value: str, /) -> Self:
            """
            Type is the type of the condition.
            """
            return self._set("type", value)

    class BuilderContext(BuilderContextBase["APIServiceCondition.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = APIServiceCondition.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for APIServiceCondition."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["APIServiceCondition", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use APIServiceCondition.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    last_transition_time: Annotated[
        Optional[apimachinery.Time], Field(alias="lastTransitionTime")
    ] = None
    """
    Last time the condition transitioned from one status to another.
    """
    message: Optional[str] = None
    """
    Human-readable message indicating details about last transition.
    """
    reason: Optional[str] = None
    """
    Unique, one-word, CamelCase reason for the condition's last transition.
    """
    status: str
    """
    Status is the status of the condition. Can be True, False, Unknown.
    """
    type: str
    """
    Type is the type of the condition.
    """


class APIServiceStatus(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["APIServiceStatus"]:
            return APIServiceStatus

        def build(self) -> "APIServiceStatus":
            return APIServiceStatus(**self._attrs)

        @overload
        def conditions(
            self, value_or_callback: List[APIServiceCondition], /
        ) -> "APIServiceStatus.Builder": ...

        @overload
        def conditions(
            self,
            value_or_callback: Callable[
                [GenericListBuilder[APIServiceCondition, APIServiceCondition.Builder]],
                GenericListBuilder[APIServiceCondition, APIServiceCondition.Builder]
                | List[APIServiceCondition],
            ],
            /,
        ) -> "APIServiceStatus.Builder": ...

        @overload
        def conditions(
            self, value_or_callback: Never = ...
        ) -> ListBuilderContext[APIServiceCondition.Builder]: ...

        def conditions(self, value_or_callback=None, /):
            """
            Current service state of apiService.
            """
            if self._in_context and value_or_callback is None:
                context = ListBuilderContext[APIServiceCondition.Builder]()
                context._parent_builder = self
                context._field_name = "conditions"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(APIServiceCondition.list_builder())
                if isinstance(output, GenericListBuilder):
                    value = output.build()
                else:
                    value = output
            return self._set("conditions", value)

    class BuilderContext(BuilderContextBase["APIServiceStatus.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = APIServiceStatus.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for APIServiceStatus."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["APIServiceStatus", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use APIServiceStatus.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    conditions: Optional[List[APIServiceCondition]] = None
    """
    Current service state of apiService.
    """


class ServiceReference(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ServiceReference"]:
            return ServiceReference

        def build(self) -> "ServiceReference":
            return ServiceReference(**self._attrs)

        def name(self, value: Optional[str], /) -> Self:
            """
            Name is the name of the service
            """
            return self._set("name", value)

        def namespace(self, value: Optional[str], /) -> Self:
            """
            Namespace is the namespace of the service
            """
            return self._set("namespace", value)

        def port(self, value: Optional[int], /) -> Self:
            """
            If specified, the port on the service that hosting webhook. Default to 443 for backward compatibility. `port` should be a valid port number (1-65535, inclusive).
            """
            return self._set("port", value)

    class BuilderContext(BuilderContextBase["ServiceReference.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ServiceReference.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ServiceReference."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ServiceReference", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ServiceReference.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    name: Optional[str] = None
    """
    Name is the name of the service
    """
    namespace: Optional[str] = None
    """
    Namespace is the namespace of the service
    """
    port: Optional[int] = None
    """
    If specified, the port on the service that hosting webhook. Default to 443 for backward compatibility. `port` should be a valid port number (1-65535, inclusive).
    """


class APIServiceSpec(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["APIServiceSpec"]:
            return APIServiceSpec

        def build(self) -> "APIServiceSpec":
            return APIServiceSpec(**self._attrs)

        def ca_bundle(self, value: Optional[str], /) -> Self:
            """
            CABundle is a PEM encoded CA bundle which will be used to validate an API server's serving certificate. If unspecified, system trust roots on the apiserver are used.
            """
            return self._set("ca_bundle", value)

        def group(self, value: Optional[str], /) -> Self:
            """
            Group is the API group name this server hosts
            """
            return self._set("group", value)

        def group_priority_minimum(self, value: int, /) -> Self:
            """
            GroupPriorityMinimum is the priority this group should have at least. Higher priority means that the group is preferred by clients over lower priority ones. Note that other versions of this group might specify even higher GroupPriorityMinimum values such that the whole group gets a higher priority. The primary sort is based on GroupPriorityMinimum, ordered highest number to lowest (20 before 10). The secondary sort is based on the alphabetical comparison of the name of the object.  (v1.bar before v1.foo) We'd recommend something like: *.k8s.io (except extensions) at 18000 and PaaSes (OpenShift, Deis) are recommended to be in the 2000s
            """
            return self._set("group_priority_minimum", value)

        def insecure_skip_tls_verify(self, value: Optional[bool], /) -> Self:
            """
            InsecureSkipTLSVerify disables TLS certificate verification when communicating with this server. This is strongly discouraged.  You should use the CABundle instead.
            """
            return self._set("insecure_skip_tls_verify", value)

        @overload
        def service(
            self, value_or_callback: Optional[ServiceReference], /
        ) -> "APIServiceSpec.Builder": ...

        @overload
        def service(
            self,
            value_or_callback: Callable[
                [ServiceReference.Builder], ServiceReference.Builder | ServiceReference
            ],
            /,
        ) -> "APIServiceSpec.Builder": ...

        @overload
        def service(self, value_or_callback: Never = ...) -> "ServiceReference.BuilderContext": ...

        def service(self, value_or_callback=None, /):
            """
            Service is a reference to the service for this API server.  It must communicate on port 443. If the Service is nil, that means the handling for the API groupversion is handled locally on this server. The call will simply delegate to the normal handler chain to be fulfilled.
            """
            if self._in_context and value_or_callback is None:
                context = ServiceReference.BuilderContext()
                context._parent_builder = self
                context._field_name = "service"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ServiceReference.builder())
                if isinstance(output, ServiceReference.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("service", value)

        def version(self, value: Optional[str], /) -> Self:
            """
            Version is the API version this server hosts.  For example, "v1"
            """
            return self._set("version", value)

        def version_priority(self, value: int, /) -> Self:
            """
            VersionPriority controls the ordering of this API version inside of its group.  Must be greater than zero. The primary sort is based on VersionPriority, ordered highest to lowest (20 before 10). Since it's inside of a group, the number can be small, probably in the 10s. In case of equal version priorities, the version string will be used to compute the order inside a group. If the version string is "kube-like", it will sort above non "kube-like" version strings, which are ordered lexicographically. "Kube-like" versions start with a "v", then are followed by a number (the major version), then optionally the string "alpha" or "beta" and another number (the minor version). These are sorted first by GA > beta > alpha (where GA is a version with no suffix such as beta or alpha), and then by comparing major version, then minor version. An example sorted list of versions: v10, v2, v1, v11beta2, v10beta3, v3beta1, v12alpha1, v11alpha2, foo1, foo10.
            """
            return self._set("version_priority", value)

    class BuilderContext(BuilderContextBase["APIServiceSpec.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = APIServiceSpec.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for APIServiceSpec."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["APIServiceSpec", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use APIServiceSpec.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    ca_bundle: Annotated[Optional[str], Field(alias="caBundle")] = None
    """
    CABundle is a PEM encoded CA bundle which will be used to validate an API server's serving certificate. If unspecified, system trust roots on the apiserver are used.
    """
    group: Optional[str] = None
    """
    Group is the API group name this server hosts
    """
    group_priority_minimum: Annotated[int, Field(alias="groupPriorityMinimum")]
    """
    GroupPriorityMinimum is the priority this group should have at least. Higher priority means that the group is preferred by clients over lower priority ones. Note that other versions of this group might specify even higher GroupPriorityMinimum values such that the whole group gets a higher priority. The primary sort is based on GroupPriorityMinimum, ordered highest number to lowest (20 before 10). The secondary sort is based on the alphabetical comparison of the name of the object.  (v1.bar before v1.foo) We'd recommend something like: *.k8s.io (except extensions) at 18000 and PaaSes (OpenShift, Deis) are recommended to be in the 2000s
    """
    insecure_skip_tls_verify: Annotated[Optional[bool], Field(alias="insecureSkipTLSVerify")] = None
    """
    InsecureSkipTLSVerify disables TLS certificate verification when communicating with this server. This is strongly discouraged.  You should use the CABundle instead.
    """
    service: Optional[ServiceReference] = None
    """
    Service is a reference to the service for this API server.  It must communicate on port 443. If the Service is nil, that means the handling for the API groupversion is handled locally on this server. The call will simply delegate to the normal handler chain to be fulfilled.
    """
    version: Optional[str] = None
    """
    Version is the API version this server hosts.  For example, "v1"
    """
    version_priority: Annotated[int, Field(alias="versionPriority")]
    """
    VersionPriority controls the ordering of this API version inside of its group.  Must be greater than zero. The primary sort is based on VersionPriority, ordered highest to lowest (20 before 10). Since it's inside of a group, the number can be small, probably in the 10s. In case of equal version priorities, the version string will be used to compute the order inside a group. If the version string is "kube-like", it will sort above non "kube-like" version strings, which are ordered lexicographically. "Kube-like" versions start with a "v", then are followed by a number (the major version), then optionally the string "alpha" or "beta" and another number (the minor version). These are sorted first by GA > beta > alpha (where GA is a version with no suffix such as beta or alpha), and then by comparing major version, then minor version. An example sorted list of versions: v10, v2, v1, v11beta2, v10beta3, v3beta1, v12alpha1, v11alpha2, foo1, foo10.
    """


class APIService(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["APIService"]:
            return APIService

        def build(self) -> "APIService":
            return APIService(**self._attrs)

        def api_version(self, value: Optional[Literal["apiregistration.k8s.io/v1"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["APIService"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "APIService.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "APIService.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def spec(self, value_or_callback: Optional[APIServiceSpec], /) -> "APIService.Builder": ...

        @overload
        def spec(
            self,
            value_or_callback: Callable[
                [APIServiceSpec.Builder], APIServiceSpec.Builder | APIServiceSpec
            ],
            /,
        ) -> "APIService.Builder": ...

        @overload
        def spec(self, value_or_callback: Never = ...) -> "APIServiceSpec.BuilderContext": ...

        def spec(self, value_or_callback=None, /):
            """
            Spec contains information for locating and communicating with a server
            """
            if self._in_context and value_or_callback is None:
                context = APIServiceSpec.BuilderContext()
                context._parent_builder = self
                context._field_name = "spec"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(APIServiceSpec.builder())
                if isinstance(output, APIServiceSpec.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("spec", value)

        @overload
        def status(
            self, value_or_callback: Optional[APIServiceStatus], /
        ) -> "APIService.Builder": ...

        @overload
        def status(
            self,
            value_or_callback: Callable[
                [APIServiceStatus.Builder], APIServiceStatus.Builder | APIServiceStatus
            ],
            /,
        ) -> "APIService.Builder": ...

        @overload
        def status(self, value_or_callback: Never = ...) -> "APIServiceStatus.BuilderContext": ...

        def status(self, value_or_callback=None, /):
            """
            Status contains derived information about an API server
            """
            if self._in_context and value_or_callback is None:
                context = APIServiceStatus.BuilderContext()
                context._parent_builder = self
                context._field_name = "status"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(APIServiceStatus.builder())
                if isinstance(output, APIServiceStatus.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("status", value)

    class BuilderContext(BuilderContextBase["APIService.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = APIService.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for APIService."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["APIService", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use APIService.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[
        Optional[Literal["apiregistration.k8s.io/v1"]], Field(alias="apiVersion")
    ] = "apiregistration.k8s.io/v1"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["APIService"]] = "APIService"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata
    """
    spec: Optional[APIServiceSpec] = None
    """
    Spec contains information for locating and communicating with a server
    """
    status: Optional[APIServiceStatus] = None
    """
    Status contains derived information about an API server
    """


APIServiceList = ResourceList["APIService"]



# Generated by cloudcoil-model-codegen v0.3.0
# DO NOT EDIT

from __future__ import annotations

from typing import (
    Annotated,
    Callable,
    Literal,
    Optional,
    Type,
    overload,
)

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil.pydantic import (
    BaseModel,
    BaseModelBuilder,
    BuilderContextBase,
    GenericListBuilder,
    Never,
    Self,
)
from cloudcoil.resources import Resource, ResourceList


class ClusterTrustBundleSpec(BaseModel):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ClusterTrustBundleSpec"]:
            return ClusterTrustBundleSpec

        def build(self) -> "ClusterTrustBundleSpec":
            return ClusterTrustBundleSpec(**self._attrs)

        def signer_name(self, value: Optional[str], /) -> Self:
            """
            signerName indicates the associated signer, if any.

            In order to create or update a ClusterTrustBundle that sets signerName, you must have the following cluster-scoped permission: group=certificates.k8s.io resource=signers resourceName=<the signer name> verb=attest.

            If signerName is not empty, then the ClusterTrustBundle object must be named with the signer name as a prefix (translating slashes to colons). For example, for the signer name `example.com/foo`, valid ClusterTrustBundle object names include `example.com:foo:abc` and `example.com:foo:v1`.

            If signerName is empty, then the ClusterTrustBundle object's name must not have such a prefix.

            List/watch requests for ClusterTrustBundles can filter on this field using a `spec.signerName=NAME` field selector.
            """
            return self._set("signer_name", value)

        def trust_bundle(self, value: str, /) -> Self:
            """
            trustBundle contains the individual X.509 trust anchors for this bundle, as PEM bundle of PEM-wrapped, DER-formatted X.509 certificates.

            The data must consist only of PEM certificate blocks that parse as valid X.509 certificates.  Each certificate must include a basic constraints extension with the CA bit set.  The API server will reject objects that contain duplicate certificates, or that use PEM block headers.

            Users of ClusterTrustBundles, including Kubelet, are free to reorder and deduplicate certificate blocks in this file according to their own logic, as well as to drop PEM block headers and inter-block data.
            """
            return self._set("trust_bundle", value)

    class BuilderContext(BuilderContextBase["ClusterTrustBundleSpec.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ClusterTrustBundleSpec.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ClusterTrustBundleSpec."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ClusterTrustBundleSpec", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ClusterTrustBundleSpec.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    signer_name: Annotated[Optional[str], Field(alias="signerName")] = None
    """
    signerName indicates the associated signer, if any.

    In order to create or update a ClusterTrustBundle that sets signerName, you must have the following cluster-scoped permission: group=certificates.k8s.io resource=signers resourceName=<the signer name> verb=attest.

    If signerName is not empty, then the ClusterTrustBundle object must be named with the signer name as a prefix (translating slashes to colons). For example, for the signer name `example.com/foo`, valid ClusterTrustBundle object names include `example.com:foo:abc` and `example.com:foo:v1`.

    If signerName is empty, then the ClusterTrustBundle object's name must not have such a prefix.

    List/watch requests for ClusterTrustBundles can filter on this field using a `spec.signerName=NAME` field selector.
    """
    trust_bundle: Annotated[str, Field(alias="trustBundle")]
    """
    trustBundle contains the individual X.509 trust anchors for this bundle, as PEM bundle of PEM-wrapped, DER-formatted X.509 certificates.

    The data must consist only of PEM certificate blocks that parse as valid X.509 certificates.  Each certificate must include a basic constraints extension with the CA bit set.  The API server will reject objects that contain duplicate certificates, or that use PEM block headers.

    Users of ClusterTrustBundles, including Kubelet, are free to reorder and deduplicate certificate blocks in this file according to their own logic, as well as to drop PEM block headers and inter-block data.
    """


class ClusterTrustBundle(Resource):
    class Builder(BaseModelBuilder):
        @property
        def cls(self) -> Type["ClusterTrustBundle"]:
            return ClusterTrustBundle

        def build(self) -> "ClusterTrustBundle":
            return ClusterTrustBundle(**self._attrs)

        def api_version(self, value: Optional[Literal["certificates.k8s.io/v1alpha1"]], /) -> Self:
            """
            APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
            """
            return self._set("api_version", value)

        def kind(self, value: Optional[Literal["ClusterTrustBundle"]], /) -> Self:
            """
            Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
            """
            return self._set("kind", value)

        @overload
        def metadata(
            self, value_or_callback: Optional[apimachinery.ObjectMeta], /
        ) -> "ClusterTrustBundle.Builder": ...

        @overload
        def metadata(
            self,
            value_or_callback: Callable[
                [apimachinery.ObjectMeta.Builder],
                apimachinery.ObjectMeta.Builder | apimachinery.ObjectMeta,
            ],
            /,
        ) -> "ClusterTrustBundle.Builder": ...

        @overload
        def metadata(
            self, value_or_callback: Never = ...
        ) -> "apimachinery.ObjectMeta.BuilderContext": ...

        def metadata(self, value_or_callback=None, /):
            """
            metadata contains the object metadata.
            """
            if self._in_context and value_or_callback is None:
                context = apimachinery.ObjectMeta.BuilderContext()
                context._parent_builder = self
                context._field_name = "metadata"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(apimachinery.ObjectMeta.builder())
                if isinstance(output, apimachinery.ObjectMeta.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("metadata", value)

        @overload
        def spec(
            self, value_or_callback: ClusterTrustBundleSpec, /
        ) -> "ClusterTrustBundle.Builder": ...

        @overload
        def spec(
            self,
            value_or_callback: Callable[
                [ClusterTrustBundleSpec.Builder],
                ClusterTrustBundleSpec.Builder | ClusterTrustBundleSpec,
            ],
            /,
        ) -> "ClusterTrustBundle.Builder": ...

        @overload
        def spec(
            self, value_or_callback: Never = ...
        ) -> "ClusterTrustBundleSpec.BuilderContext": ...

        def spec(self, value_or_callback=None, /):
            """
            spec contains the signer (if any) and trust anchors.
            """
            if self._in_context and value_or_callback is None:
                context = ClusterTrustBundleSpec.BuilderContext()
                context._parent_builder = self
                context._field_name = "spec"
                return context

            value = value_or_callback
            if callable(value_or_callback):
                output = value_or_callback(ClusterTrustBundleSpec.builder())
                if isinstance(output, ClusterTrustBundleSpec.Builder):
                    value = output.build()
                else:
                    value = output
            return self._set("spec", value)

    class BuilderContext(BuilderContextBase["ClusterTrustBundle.Builder"]):
        def model_post_init(self, __context) -> None:
            self._builder = ClusterTrustBundle.Builder()
            self._builder._in_context = True
            self._parent_builder = None
            self._field_name = None

    @classmethod
    def builder(cls) -> Builder:
        return cls.Builder()

    @classmethod
    def new(cls) -> BuilderContext:
        """Creates a new context manager builder for ClusterTrustBundle."""
        return cls.BuilderContext()

    class ListBuilder(GenericListBuilder["ClusterTrustBundle", Builder]):
        def __init__(self):
            raise NotImplementedError(
                "This class is not meant to be instantiated. Use ClusterTrustBundle.list_builder() instead."
            )

    @classmethod
    def list_builder(cls) -> ListBuilder:
        return GenericListBuilder[cls, cls.Builder]()  # type: ignore

    api_version: Annotated[
        Optional[Literal["certificates.k8s.io/v1alpha1"]], Field(alias="apiVersion")
    ] = "certificates.k8s.io/v1alpha1"
    """
    APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources
    """
    kind: Optional[Literal["ClusterTrustBundle"]] = "ClusterTrustBundle"
    """
    Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds
    """
    metadata: Optional[apimachinery.ObjectMeta] = None
    """
    metadata contains the object metadata.
    """
    spec: ClusterTrustBundleSpec
    """
    spec contains the signer (if any) and trust anchors.
    """


ClusterTrustBundleList = ResourceList["ClusterTrustBundle"]
